#
# list.py - DITrack 'list' command
#
# Copyright (c) 2006-2007 The DITrack Project, www.ditrack.org.
#
# $Id: list.py 1846 2007-08-04 11:25:09Z gli $
# $HeadURL: https://svn.xiolabs.com/ditrack/src/tags/0.6/DITrack/Command/list.py $
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  * Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import os

# DITrack modules
import DITrack.DB.Common
import DITrack.DB.Exceptions
import DITrack.Client
import DITrack.Command.generic

class _FallbackDict:
    """
    A wrapper (fallback) dictionary-like class allowing to fetch info header 
    fields and fall back to default values if there is no such header present.
    """
    def __getitem__(self, key):
        if key == "id":
            return "%s" % self.id
        elif key in self.info:
            return self.info[key]
        else:
            return "-";

    def __init__(self, id, info):
        self.id = id
        self.info = info

class Handler(DITrack.Command.generic.Handler):
    canonical_name = "list"
    description = """List issues, possibly filtering.
usage: %s [FILTER...]""" % canonical_name

    def run(self, opts, globals):
        self.check_options(opts)

        db = DITrack.Util.common.open_db(globals, opts)

        client = DITrack.Client.Client(db)

        filters = []


        for e in opts.fixed[1:]:
            try:
                f = DITrack.DB.Common.Filter(e, globals.username)
            except DITrack.DB.Exceptions.FilterIsPredefinedError:
                if e in db.cfg.filters:
                    f = db.cfg.filters[e]
                else:
                    DITrack.Util.common.err(
                        "ERROR: '%s' is not a predefined filter" % e
                    )

            except DITrack.DB.Exceptions.FilterExpressionError:
                DITrack.Util.common.err(
                    "ERROR: '%s' contains a syntax error" % e
                )

            filters.append(f)
       
        try:
            if opts.var.has_key("listing_format"):
                format = db.cfg.listing_format[opts.var["listing_format"]]
            else:
                format = db.cfg.listing_format["default"]
        except DITrack.DB.Exceptions.InvalidListingFormatError, key:
            DITrack.Util.common.err(
                "ERROR: Unknown listing format '%s'" % key
            )

        for id, issue in client.issues(filters):
            print format % _FallbackDict(id, issue.info)
